/***********************************
 Bridging Graphics:
 provides functionality for simple graphics

 Copyright (c) 2006-2007, Lori Scarlatos

 Module Name:
	bg.h

 Module Purpose:
	function declarations and constant/type definitions
***********************************/

#ifndef __bg_h_
#ifndef __BG_H__

#define __bg_h_
#define __BG_H__

// Windows.h is needed only if running in Windows
#include <windows.h>

#include <GL/gl.h>
#include <GL/glut.h>

#include <string>
using namespace std;

/***** CONSTANTS *****/
// Color constants

enum BGcolors {BG_BLACK, BG_WHITE, BG_GREY,
               BG_RED, BG_GREEN, BG_BLUE,
               BG_YELLOW, BG_CYAN, BG_MAGENTA,
               BG_ORANGE, BG_PINK, BG_BROWN, BG_PURPLE};

// Window types

enum BGwindowtypes { BG_SQUARE, BG_RECTANGLE, BG_FULL };

// Callback types

enum BGcallbacks { BG_KEY, BG_SPECIAL_KEY, BG_IDLE }; 

// Image types

enum BGimagetypes { BG_GREY_IMAGE, BG_HUE_IMAGE, BG_COLOR_IMAGE, BG_RGB_IMAGE, BG_RGBA_IMAGE, BG_GREY_ALPHA_IMAGE };

/***** FUNCTION DECLARATIONS *****/

// Window functions

extern void bgCreateWindow (void);
extern void bgCreateWindow (enum BGwindowtypes);
extern float bgGetWindowWidth (void);
extern float bgGetWindowHeight (void);
extern float bgGetWindowLeft (void);
extern float bgGetWindowRight (void);
extern float bgGetWindowBottom (void);
extern float bgGetWindowTop (void);

void bgMainLoop (void);
void bgPause (float seconds);

// Color functions

void bgSetColor (enum BGcolors color_name);
void bgSetBackgroundColor (enum BGcolors color_name);
void bgSetHue (int hue_value);         // angle on color circle, 0..360
void bgSetGrey (float grey_value);     // brightness, 0..1
void bgSetBackgroundRGB (float red_value, float green_value, float blue_value);
void bgSetRGB (float red_value, float green_value, float blue_value);
void bgSetRGBbyte (unsigned char red_value, unsigned char green_value, unsigned char blue_value);

// Drawing functions

void bgClearScreen (void);

void bgMoveTo (float x, float y);
void bgDrawTo (float x, float y);
void bgDrawPoint (float x, float y);
void bgDrawBox (float x1, float y1, float x2, float y2);
void bgDrawTriangle (float x1, float y1, float x2, float y2, float x3, float y3);
void bgDrawCircle (float x1, float y1, float radius);

void bgUpdateDisplay (void);

// Working with Images
/*
int bgReadImage (char *filename, int *width, int *height, unsigned char **image);  // PGM file only
void bgWriteImage (char *filename);  // dumps screen to PPM file
void bgDrawGreyImage (int rows, int columns, unsigned char *image);
void bgDrawHueImage (int rows, int columns, int *image);
void bgDrawColorImage (int rows, int columns, enum BGcolors *image);
void bgDrawPPMatXY (float x, float y, float width, int rows, int columns, unsigned char *image);
void bgDrawPGMatXY (float x, float y, float width, int rows, int columns, unsigned char *image);
*/

class BGimage {
public:
	BGimage (char *filename, bool white_is_transparent);  // get image from a file
	BGimage (int num_of_rows, int num_of_cols, enum BGimagetypes imagetype); // construct empty image
	unsigned char *image; // array of pixel values
	void draw (void);	  // fills the space with the image
	void draw (float x, float y, float width); // draws the image at given location, scaled uniformly to given width
	int rows (void) { return nrows; }
	int columns (void) { return ncols; }
	int depth (void) { return nbytes; }
private:
	int nrows, ncols;	// number of rows & columns in image can't change
	int nbytes;			// number of bytes per pixel: 1=luminance (grey) or hue, 2=LA 3=RGB, 4=RGBA
	enum BGimagetypes type; // type of the image
};

// Callback functions

void bgEnableCallback(enum BGcallbacks);
void bgDisableCallback(enum BGcallbacks);

void bgKeyFunction (unsigned char key, int x, int y);
void bgSpecialKeyFunction (int key, int x, int y);
void bgIdleFunction (void);

// Text writing

void bgWriteString(float x, float y, string s);

#endif /* __BG_H__ */
#endif /* __bg_h_ */
